
--[[

The complete parsing function. Since the order of text in the combat log is likely to differ in
different languages, this entire function is included  in the localization class.

The function is given an input line from the combat log, and parses that line to determine its meaning.
The final step in each section is to return the extracted values. These lines need not be updated, but
the correct info must be extracted into the relevant fields.

Note the use of numeric values for avoids, critical hits, dmg types, and morale/power. These numbers
correspond to the orderings in the four lists directly above this comment block.

You will need to know something about lua pattern matching and/or regex's to attempt to translate this
section. See http://www.lua.org/pil/20.2.html for more details.

]]--

L["Parse"] = function(line)

	-- 1) Damage line ---
	
	local initiatorName,avoidAndCrit,skillName,targetNameAmountAndType = string.match(line,"^(.*) gelang ein (.*)Treffer mit (.*) gegen (.*)%.$");
	
	if (initiatorName ~= nil) then
		
		initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Dem ","");
		skillName = string.gsub(skillName, "\"", "");
		
		
		
		local avoidType =
			string.match(avoidAndCrit,"^teilweise geblockter") and 8 or
			string.match(avoidAndCrit,"^teilweise parierter") and 9 or
			string.match(avoidAndCrit,"^teilweise ausgewichener") and 10 or 1;
		local critType =
			string.match(avoidAndCrit,"kritischer $") and 2 or
			string.match(avoidAndCrit,"zerst\195\182rerischer $") and 3 or 1;
		
		local targetName,amount,dmgType,moralePower = string.match(targetNameAmountAndType,"^(.*) f\195\188r ([%d,]*) (.*) auf (.*)$");

	-- damage was absorbed
		if targetName == nil then
			targetName = string.gsub(targetNameAmountAndType,"^den ","");
			targetName = string.gsub(targetNameAmountAndType,"^die ","");
			amount = 0;
			dmgType = 10;
			moralePower = 3;
		-- some damage was dealt
		else
			targetName = string.gsub(targetName,"^den ","");
			targetName = string.gsub(targetName,"^die ","");
			amount = string.gsub(amount,",","")+0;
			-- note there may be no damage type
			dmgType = 
				dmgType == "Punkte Schaden des Typs \"Allgemein\""  and 1 or
				dmgType == "Punkte Schaden des Typs \"Feuer\"" and 2 or
				dmgType == "Punkte Schaden des Typs \"Blitz\"" and 3 or
				dmgType == "Punkte Schaden des Typs \"Frost\"" and 4 or
				dmgType == "Punkte Schaden des Typs \"S\195\164ure\"" and 5 or
				dmgType == "Punkte Schaden des Typs \"Schatten\"" and 6 or
				dmgType == "Punkte Schaden des Typs \"Licht\"" and 7 or
				dmgType == "Punkte Schaden des Typs \"Beleriand-Art\"" and 8 or
				dmgType == "Punkte Schaden des Typs \"Westernis-Art\"" and 9 or
				dmgType == "Punkte Schaden des Typs \"Uralte Zwergenart\"" and 10 or 11;
			moralePower = (moralePower == "Moral" and 1 or moralePower == "Kraft" and 2 or 3);
		end
		
		-- Currently ignores damage to power
		if (moralePower == 2) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,amount,avoidType,critType,dmgType;
	end
	
	-- 2) Heal line --
	--     (note the distinction with which self heals are now handled)
	--     (note we consider the case of heals of zero magnitude, even though they presumably never occur)
	
	---HEAL auf andere
	---Der Ambur wandte "Heilung" mit "Herz der Gefhrten" auf die Amtholdiel an, was 1,049 Punkte Moral wiederherstellte.
	---Amtholdiel applied a heal with Inner Flame to the Blood-raven restoring 91 points to Morale.
	local initiatorName,crit,skillName,targetName,amount,type = string.match(line,"^(.*) wandte \"(.*)Heilung\" mit (.*) auf (.*) an, was ([%d,]*) Punkte (.*) wiederherstellte.");
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Die ","");
		local critType =
			crit == "kritische " and 2 or
			crit == "zerst\195\182rerische " and 3 or 1;
		targetName = string.gsub(targetName,"^die ","");
		targetName = string.gsub(targetName,"^den ","");
		skillName = string.gsub(skillName, "\"", "");
		amount = string.gsub(amount,",","")+0;
		moralePower = (type == "Moral" and 1 or type == "Kraft" and 2 or 3);
		
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end

	--- Heal was absorbed (eg: incoming healing = 0%?)
	if moralePower == 3 then
		targetName = string.gsub(targetName,"^den ","");
		targetName = string.gsub(targetName,"^die ","");
		targetName = string.gsub(targetName,"^der ","");
		targetName = string.gsub(targetName,"^dem ","");
		amount = 0;
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end
	
	---HEAL auf sich selbst
	---Innere Flamme verursacht bei der Amtholdiel "Heilung" und stellt 97 Punkte Moral wieder her.
	---Sign of Power: Istari Heal applied a heal to Amtholdiel restoring 987 points to Morale.
	local skillName,targetName,crit,amount,type = string.match(line,"^(.*) verursacht bei (.*) \"(.*)Heilung\" und stellt ([%d,]*) Punkte (.*) wieder her.");
	if (skillName ~= nil) then
		skillName = string.gsub(skillName, "\"", "");
		targetName = string.gsub(targetName,"^der ","");
		targetName = string.gsub(targetName,"^dem ","");
		local initiatorName = targetName; 
		local critType =
			crit == "kritische " and 2 or
			crit == "zerst\195\182rerische " and 3 or 1;
		amount = string.gsub(amount,",","")+0;
		moralePower = (type == "Moral" and 1 or type == "Kraft" and 2 or 3);
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end

	-- 3) Buff line --
	---Die Amtholdiel wandte "Vorteil" mit "Verbesserung: Zeichen der Wildnis: Wut" auf Geist der Natur an.
	local initiatorName,skillName,targetName = string.match(line,"^(.*) wandte \"Vorteil\" mit (.*) auf (.*) an%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^Die ","");
		initiatorName = string.gsub(initiatorName,"^Der ","");
		targetName = string.gsub(targetName,"^den ","");
		targetName = string.gsub(targetName,"^die ","");
		
		-- Update
		return 14,initiatorName,targetName,skillName;
	end
	
	-- 4) Avoid line --
	---bungspuppe wollte die Amtholdiel mit "DNT - Training Dummy Swing" treffen, aber !!!!!!!!!!SIE!!!!!!!! konterte den Versuch mit "Parieren".
	---Der Angmarim-Kundschafter wollte die Phillipa mit "ein schwacher Fernkampfangriff" treffen, aber sie konterte den Versuch mit "Ausweichen".
	---Absorbieren,Ausweichen, blocken und co...
	local initiatorName,targetName,skillName,erSie,avoidType = string.match(line,"^(.*) wollte (.*) mit (.*) treffen\, aber (.*) konterte den Versuch mit (.*).");
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Die ","");
		targetName = string.gsub(targetName,"^den ","");
		targetName = string.gsub(targetName,"^die ","");
		skillName = string.gsub(skillName, "\"", "");
		avoidType = string.gsub(avoidType, "\"", "");
		avoidType = 
				string.match(avoidType,"Blocken") and 5 or
				string.match(avoidType,"Parieren") and 6 or
				string.match(avoidType,"Ausweichen") and 7 or
				string.match(avoidType,"Widerstehen") and 4 or
				string.match(avoidType,"Immunit\195\164t") and 3 or 1;
		
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
	
	---Die Amtholdiel wollte Ghsh-olog der Weien Hand mit "Lhmung verursachen" treffen, aber konterte den Versuch mit "Immunitt".
	local initiatorName,targetName,skillName,avoidType = string.match(line,"^(.*) wollte (.*) mit (.*) treffen\, aber konterte den Versuch mit (.*).");
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Die ","");
		targetName = string.gsub(targetName,"^den ","");
		targetName = string.gsub(targetName,"^die ","");
		skillName = string.gsub(skillName, "\"", "");
		avoidType = string.gsub(avoidType, "\"", "");
		avoidType = 
				string.match(avoidType,"Blocken") and 5 or
				string.match(avoidType,"Parieren") and 6 or
				string.match(avoidType,"Ausweichen") and 7 or
				string.match(avoidType,"Widerstehen") and 4 or
				string.match(avoidType,"Immunit\195\164t") and 3 or 1;
		
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
	
	---Verfehlen
	---The Training-dummy missed trying to use DNT - Training Dummy Swing on Amtholdiel
	local initiatorName,targetName,skillName = string.match(line,"^(.*) verfehlte (.*) mit (.*).");
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Die ","");
		targetName = string.gsub(targetName,"^den ","");
		targetName = string.gsub(targetName,"^die ","");
		skillName = string.gsub(skillName, "\"", "");
		avoidType = 2;
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		-- Update
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
	
	-- 5) Reflect line --
	---Die Amtholdiel reflektierte 27 Punkte Schaden des Typs "Licht" auf Moral von Ghsh-hai der Weien Hand.

	local initiatorName,amount,dmgType,targetName = string.match(line,"^(.*) reflektierte ([%d,]*) Punkte (.*) Moral von (.*)%.$");
	if (initiatorName ~= nil) then
		local skillName = "Reflektiert";
		initiatorName = string.gsub(initiatorName,"^Die ","");
		initiatorName = string.gsub(initiatorName,"^Der ","");
		targetName = string.gsub(targetName,"^der ","");
		targetName = string.gsub(targetName,"^dem ","");
		targetName = string.gsub(targetName,"^die ","");
		amount = string.gsub(amount,",","")+0;

		dmgType = string.match(dmgType,"^Schaden des Typs (.*) auf$");
		-- a damage reflect
		if (dmgType ~= nil) then
			
			dmgType = 
				dmgType == "\"Allgemein\""  and 1 or
				dmgType == "\"Feuer\"" and 2 or
				dmgType == "\"Blitz\"" and 3 or
				dmgType == "\"Frost\"" and 4 or
				dmgType == "\"S\195\164ure\"" and 5 or
				dmgType == "\"Schatten\"" and 6 or
				dmgType == "\"Licht\"" and 7 or
				dmgType == "\"Beleriand-Art\"" and 8 or
				dmgType == "\"Westernis-Art\"" and 9 or
				dmgType == "\"Uralte Zwergenart\"" and 10 or 11;
				
			-- Update
			return 1,initiatorName,targetName,skillName,amount,1,1,dmgType;
		else
		return 3,initiatorName,targetName,skillName,amount,1;
		
		end
	end
	
	--- a heal reflect
	---Der Murk reflektierte 44 Punkte zur Wiederherstellung von Moral von der Rasfiel.
	---Der Gorthorog-Verteidiger reflektierte 381 Punkte zur Wiederherstellung von Moral von der Amtholdiel.
 --[[
	local initiatorName,amount,targetName = string.match(line,"^(.*) reflektierte ([%d,]*) Punkte zur Wiederherstellung von Moral von (.*)%.$");
	if (initiatorName ~= nil) then
		local skillName = "Reflect";
		initiatorName = string.gsub(initiatorName,"^Die ","");
		initiatorName = string.gsub(initiatorName,"^Der ","");
		targetName = string.gsub(targetName,"^der ","");
		targetName = string.gsub(targetName,"^dem ","");
		targetName = string.gsub(targetName,"^die ","");
		amount = string.gsub(amount,",","")+0;
				
		return 3,initiatorName,targetName,skillName,amount,1;
	end	
]]--
	
	-- 6) Interrupt line --
	---Der Buufuu wurde durch die Amtholdiel unterbochen!
	---Die Amtholdiel wurde durch die Phillipa unterbochen!
	local targetName,initiatorName = string.match(line,"^(.*) wurde durch (.*) unterbochen!");
	if (targetName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^die ","");
		initiatorName = string.gsub(initiatorName,"^den ","");
		targetName = string.gsub(targetName,"^Der ","");
		targetName = string.gsub(targetName,"^Die ","");
		
		-- Update
		return 7,initiatorName,targetName;
	end
	
	-- 7) Dispell line (corruption removal) --
	---Von ble Inspiration geheilt: Moria-Saboteur
	---Von Blutrnstige Rage geheilt: Durub-Plnderer
	local corruption, targetName = string.match(line,"Von (.*) geheilt: (.*)");
	
	if (corruption ~= nil) then
		initiatorName = player.name;
		---targetName = string.gsub(targetName,"^Den ","");
		---targetName = string.gsub(targetName,"^Die ","");
		
		-- NB: Currently ignore corruption name
		
		-- Update
		return 8,initiatorName,targetName;
	end
	
	-- 8) Defeat lines ---
	
	-- 8a) Defeat line 1 (mob or played was killed)
	local initiatorName = string.match(line,"^.* besiegt: (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^den ","");
		initiatorName = string.gsub(initiatorName,"^die ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8b) Defeat line 2 (mob died)
	local initiatorName = string.match(line,"^(.*) died%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8c) Defeat line 3 (a player was killed or died)
	local initiatorName = string.match(line,"^(.*) has been defeated%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8d) Defeat line 4 (you were killed)
	local match = string.match(line,"^.* hat Euch auer Gefecht gesetzt%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8e) Defeat line 5 (you died)
	---Ihr wurdet durch ein Missgeschick auer Gefecht gesetzt.
	local match = string.match(line,"^Ihr wurdet durch ein Missgeschick auer Gefecht gesetzt%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 9) Revive lines --
	
	-- 9a) Revive line 1 (player revived)
	local initiatorName = string.match(line,"^(.*) has been revived%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9b) Revive line 2 (player succumbed)
	---Kaisimir ist seinen Wunden erlegen.
	local initiatorName = string.match(line,"^(.*) ist .* Wunden erlegen%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^Der ","");
		initiatorName = string.gsub(initiatorName,"^Die ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9c) Revive line 3 (you were revived)
	local match = string.match(line,"^You have been revived%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9d) Revive line 4 (you succumbed)
	---Ihr erliegt Euren Verletzungen.
	local match = string.match(line,"^Ihr erliegt Euren Verletzungen%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 10) Temporary Morale line
	-- v4.0.1
	-- local amount = string.match(line,"You have lost ([%d,]*) points of temporary Morale!$");
	-- if (amount ~= nil) then
	--   -- Update
	--   
	--   
	--   return;
	-- end
	-- 
	--  (note for later: will need to also check off to see if corresponding damage was taken; always comes directly AFTER this line)
	
	
	-- if we reach here, we were unable to parse the line
	--  (note there is very little that isn't parsed; just a few combat events such as breaking CC,
	--     though some stuff is ignored, eg: dmg taken by pets)
	
end






--[[


Note: The following DOES NOT NEED TO BE ALTERED for the current version (but does need to exist).


The following is a (work-in-progress) list of specific default skills that will be tracked for
buff/debuff purposes in later versions of Combat Analysis. 

NB: If you do update this list, the skill names must identically match the text in the combat log in order for
these skills to be correctly detected.

The elements are indexed by: {Skill Name} = {Duration(s), {List of Conflicting Skill Names}}
 ... where conflicting skills are skills that get overidden by the application of this skill
			 (note this is only relevant for debuffs, buff tracking should allow us to determine overriding buffs automatically)
 ... durations are specified in seconds
			 (note that a duration of zero seconds indicates a toggle type skill)
	
]]--

-- 1) Burglar

-- a) Buffs
local BurglarBuffs = {}
-- b) Self Buffs
local BurglarSelfBuffs = {}
BurglarSelfBuffs["Touch and Go"] = {30}
-- Knives Out not included as it does not always appear in the log (if no mobs are nearby)
-- c) Debuffs
local BurglarDebuffs = {}
BurglarDebuffs["Reveal Weakness"] = {0, {}}
BurglarDebuffs["Trick: Disable"] = {30, {"Trick: Counter Defence", "Trick: Dust in the Eyes", "Trick: Enrage"}}
BurglarDebuffs["Trick: Counter Defence"] = {30, {"Trick: Disable", "Trick: Dust in the Eyes", "Trick: Enrage"}}
BurglarDebuffs["Trick: Dust in the Eyes"] = {30, {"Trick: Disable", "Trick: Counter Defence", "Trick: Enrage"}}
BurglarDebuffs["Trick: Enrage"] = {30, {"Trick: Disable", "Trick: Counter Defence", "Trick: Dust in the Eyes"}}
-- d) Crowd Control
local BurglarCrowdControl = {};
BurglarCrowdControl["Riddle"] = {30};

L["BurglarSkills"] = {BurglarBuffs,BurglarSelfBuffs,BurglarDebuffs,BurglarCrowdControl};

-- 2) Captain

-- a) Buffs
local CaptainBuffs = {}
-- b) Self Buffs
local CaptainSelfBuffs = {}
-- c) Debuffs
local CaptainDebuffs = {}
-- d) Crowd Control
local CaptainCrowdControl = {}

L["CaptainSkills"] = {CaptainBuffs,CaptainSelfBuffs,CaptainDebuffs,CaptainCrowdControl}

-- 3) Champion

-- a) Buffs
local ChampionBuffs = {}
-- b) Self Buffs
local ChampionSelfBuffs = {}
-- c) Debuffs
local ChampionDebuffs = {}
-- d) Crowd Control
local ChampionCrowdControl = {}

L["ChampionSkills"] = {ChampionBuffs,ChampionSelfBuffs,ChampionDebuffs,ChampionCrowdControl}

-- 4) Guardian

-- a) Buffs
local GuardianBuffs = {}
GuardianBuffs["Protection"] = {0}; -- NB: Need to check upgraded skill name, etc
GuardianBuffs["Protection by the sword"] = {0}; -- NB: Need to check upgraded skill name, etc
-- b) Self Buffs
local GuardianSelfBuffs = {}
GuardianSelfBuffs["Guardian's Ward"] = {10} -- NB: Need to check upgraded skill name, etc
GuardianSelfBuffs["Guardian's Ward Tactics"] = {10} -- NB: Need to check upgraded skill name, etc
-- c) Debuffs
local GuardianDebuffs = {}
GuardianDebuffs["Improved Overwhelm"] = {10, {}} -- NB: Need to check upgraded skill name, etc
-- Stagger not included due to its unreliableness
-- d) Crowd Control
local GuardianCrowdControl = {}

L["GuardianSkills"] = {GuardianBuffs,GuardianSelfBuffs,GuardianDebuffs,GuardianCrowdControl}

-- 5) Hunter

-- a) Buffs
local HunterBuffs = {}
-- b) Self Buffs
local HunterSelfBuffs = {}
-- c) Debuffs
local HunterDebuffs = {}
-- d) Crowd Control
local HunterCrowdControl = {}

L["HunterSkills"] = {HunterBuffs,HunterSelfBuffs,HunterDebuffs,HunterCrowdControl}

-- 6) Lore-Master

-- a) Buffs
local LoreMasterBuffs = {}
-- b) Self Buffs
local LoreMasterSelfBuffs = {}
-- c) Debuffs
local LoreMasterDebuffs = {}
-- d) Crowd Control
local LoreMasterCrowdControl = {}

L["LoreMasterSkills"] = {LoreMasterBuffs,LoreMasterSelfBuffs,LoreMasterDebuffs,LoreMasterCrowdControl}

-- 7) Minstrel

-- a) Buffs
local MinstrelBuffs = {}
-- b) Self Buffs
local MinstrelSelfBuffs = {}
-- c) Debuffs
local MinstrelDebuffs = {}
-- d) Crowd Control
local MinstrelCrowdControl = {}

L["MinstrelSkills"] = {MinstrelBuffs, MinstrelSelfBuffs,MinstrelDebuffs,MinstrelCrowdControl}

-- 8) Rune-Keeper

-- a) Buffs
local RuneKeeperBuffs = {}
-- b) Self Buffs
local RuneKeeperSelfBuffs = {}
-- c) Debuffs
local RuneKeeperDebuffs = {}
-- d) Crowd Control
local RuneKeeperCrowdControl = {}

L["RuneKeeperSkills"] = {RuneKeeperBuffs,RuneKeeperSelfBuffs,RuneKeeperDebuffs,RuneKeeperCrowdControl}

-- 9) Warden

-- a) Buffs
local WardenBuffs = {}
-- b) Self Buffs
local WardenSelfBuffs = {}
-- c) Debuffs
local WardenDebuffs = {}
-- d) Crowd Control
local WardenCrowdControl = {}

L["WardenSkills"] = {WardenBuffs,WardenSelfBuffs,WardenDebuffs,WardenCrowdControl}
