

--[[

The complete parsing function. Since the order of text in the combat log is likely to differ in
different languages, this entire function is included  in the localization class.

The function is given an input line from the combat log, and parses that line to determine its meaning.
The final step in each section is to return the extracted values. These lines need not be updated, but
the correct info must be extracted into the relevant fields.

Note the use of numeric values for avoids, critical hits, dmg types, and morale/power. These numbers
correspond to the orderings in the four lists directly above this comment block.

You will need to know something about lua pattern matching and/or regex's to attempt to translate this
section. See http://www.lua.org/pil/20.2.html for more details.

]]--

L["Parse"] = function(line)

	-- 1) Damage line ---
	
	local initiatorName,avoidAndCrit,skillName,targetNameAmountAndType = string.match(line,"^(.*) scored a (.*)hit with (.*) on (.*)%.$");
	
	if (initiatorName ~= nil) then
		
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		local avoidType =
			string.match(avoidAndCrit,"^partially blocked") and 8 or
			string.match(avoidAndCrit,"^partially parried") and 9 or
			string.match(avoidAndCrit,"^partially evaded") and 10 or 1;
		local critType =
			string.match(avoidAndCrit,"critical $") and 2 or
			string.match(avoidAndCrit,"devastating $") and 3 or 1;
		
		local targetName,amount,dmgType,moralePower = string.match(targetNameAmountAndType,"^(.*) for ([%d,]*) (.*)damage to (.*)$");
		-- damage was absorbed
		if targetName == nil then
			targetName = string.gsub(targetNameAmountAndType,"^The ","");
			targetName = string.gsub(targetNameAmountAndType,"^the ","");
			amount = 0;
			dmgType = 10;
			moralePower = 3;
		-- some damage was dealt
		else
			targetName = string.gsub(targetName,"^The ","");
			targetName = string.gsub(targetName,"^the ","");
			amount = string.gsub(amount,",","")+0;
			-- note there may be no damage type
			dmgType = 
				dmgType == "Common " and 1 or
				dmgType == "Fire " and 2 or
				dmgType == "Lightning " and 3 or
				dmgType == "Frost " and 4 or
				dmgType == "Acid " and 5 or
				dmgType == "Shadow " and 6 or
				dmgType == "Light " and 7 or
				dmgType == "Beleriand " and 8 or
				dmgType == "Westernesse " and 9 or
				dmgType == "Ancient Dwarf " and 10 or 11;
			moralePower = (moralePower == "Morale" and 1 or moralePower == "Power" and 2 or 3);
		end
		
		-- Currently ignores damage to power
		if (moralePower == 2) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,amount,avoidType,critType,dmgType;
	end
	
	-- 2) Heal line --
	
	--     (note the distinction with which self heals are now handled)
	--     (note we consider the case of heals of zero magnitude, even though they presumably never occur)
	local initiatorName,crit,skillNameTargetNameAmountAndType = string.match(line,"^(.*) applied a (.-)heal (.*)%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		local critType =
			crit == "critical " and 2 or
			crit == "devastating " and 3 or 1;
			
		local skillNameTargetNameAndAmount,ending = string.match(skillNameTargetNameAmountAndType,"^(.*)to (.*)$");
		local targetName,skillName,amount;
		moralePower = (ending == "Morale" and 1 or (ending == "Power" and 2 or 3));
		-- heal was absorbed (unfortunately it appears this actually shows as a "hit" instead, so we never get into the first conditional)
		if (moralePower == 3) then
			targetName = string.gsub(ending,"^The ","");
			targetName = string.gsub(ending,"^the ","");
			amount = 0;
			-- skill name will equal nil if this was a self heal
			skillName = string.match(skillNameTargetNameAndAmount,"^with (.*) $");
		-- heal applied
		else
			skillName,targetName,amount = string.match(skillNameTargetNameAndAmount,"^(.*)to (.*) restoring ([%d,]*) points? $");
			targetName = string.gsub(targetName,"^The ","");
			targetName = string.gsub(targetName,"^the ","");
			amount = string.gsub(amount,",","")+0;
			-- skill name will equal nil if this was a self heal
			skillName = string.match(skillName,"^with (.*) $");
		end
		
		-- rearrange if this was a self heal
		if (skillName == nil) then
			skillName = initiatorName;
			initiatorName = targetName;
		end
		
		-- Update
		return (moralePower == 2 and 4 or 3),initiatorName,targetName,skillName,amount,critType;
	end
	
	-- 3) Buff line --
	
	local initiatorName,skillName,targetName = string.match(line,"^(.*) applied a benefit with (.*) on (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		targetName = string.gsub(targetName,"^The ","");
		targetName = string.gsub(targetName,"^the ","");
		
		-- Update
		return 14,initiatorName,targetName,skillName;
	end
	
	-- 4) Avoid line --
	
	local initiatorNameMiss,skillName,targetNameAvoidType = string.match(line,"^(.*) to use (.*) on (.*)%.$");
	
	if (initiatorNameMiss ~= nil) then
		initiatorName = string.match(initiatorNameMiss,"^(.*) tried$");
		local targetName, avoidType;
		-- standard avoid
		if (initiatorName ~= nil) then
			initiatorName = string.gsub(initiatorName,"^The ","");
			initiatorName = string.gsub(initiatorName,"^the ","");
			targetName,avoidType = string.match(targetNameAvoidType,"^(.*) but (.*) the attempt$");
			targetName = string.gsub(targetName,"^The ","");
			targetName = string.gsub(targetName,"^the ","");
			avoidType = 
				string.match(avoidType," blocked$") and 5 or
				string.match(avoidType," parried$") and 6 or
				string.match(avoidType," evaded$") and 7 or
				string.match(avoidType," resisted$") and 4 or
				string.match(avoidType," was immune to$") and 3 or 1;
		-- miss
		else
			initiatorName = string.match(initiatorNameMiss,"^(.*) missed trying$");
			initiatorName = string.gsub(initiatorName,"^The ","");
			initiatorName = string.gsub(initiatorName,"^the ","");
			targetName = string.gsub(targetNameAvoidType,"^The ","");
			targetName = string.gsub(targetNameAvoidType,"^the ","");
			avoidType = 2;
		end
		
		-- Sanity check: must have avoided in some manner
		if (avoidType == 1) then return nil end
		
		-- Update
		return 1,initiatorName,targetName,skillName,0,avoidType,1,10;
	end
	
	-- 5) Reflect line --
	
	local initiatorName,amount,dmgType,targetName = string.match(line,"^(.*) reflected ([%d,]*) (.*) to the Morale of (.*)%.$");
	
	if (initiatorName ~= nil) then
		local skillName = "Reflect";
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		targetName = string.gsub(targetName,"^The ","");
		targetName = string.gsub(targetName,"^the ","");
		amount = string.gsub(amount,",","")+0;
		
		local dmgType = string.match(dmgType,"^(.*)damage$");
		-- a damage reflect
		if (dmgType ~= nil) then
			dmgType = 
						dmgType == "Common " and 1 or
						dmgType == "Fire " and 2 or
						dmgType == "Lightning " and 3 or
						dmgType == "Frost " and 4 or
						dmgType == "Acid " and 5 or
						dmgType == "Shadow " and 6 or
						dmgType == "Beleriand " and 7 or
						dmgType == "Westernesse " and 8 or
						dmgType == "Ancient Dwarf " and 9 or 10;
						
			-- Update
			return 1,initiatorName,targetName,skillName,amount,1,1,dmgType;
		-- a heal reflect
		else
			-- Update
			return 3,initiatorName,targetName,skillName,amount,1;
		end
	end
	
	-- 6) Interrupt line --
	
	local targetName, initiatorName = string.match(line,"^(.*) was interrupted by (.*)!$");
	
	if (targetName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		targetName = string.gsub(targetName,"^The ","");
		targetName = string.gsub(targetName,"^the ","");
		
		-- Update
		return 7,initiatorName,targetName;
	end
	
	-- 7) Dispell line (corruption removal) --
	
	local corruption, targetName = string.match(line,"You have dispelled (.*) from (.*)%.$");
	
	if (corruption ~= nil) then
		initiatorName = player.name;
		targetName = string.gsub(targetName,"^The ","");
		targetName = string.gsub(targetName,"^the ","");
		
		-- NB: Currently ignore corruption name
		
		-- Update
		return 8,initiatorName,targetName;
	end
	
	-- 8) Defeat lines ---
	
	-- 8a) Defeat line 1 (mob or played was killed)
	local initiatorName = string.match(line,"^.* defeated (.*)%.$");
	
	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8b) Defeat line 2 (mob died)
	local initiatorName = string.match(line,"^(.*) died%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8c) Defeat line 3 (a player was killed or died)
	local initiatorName = string.match(line,"^(.*) has been defeated%.$");

	if (initiatorName ~= nil) then
		initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8d) Defeat line 4 (you were killed)
	local match = string.match(line,"^.* incapacitated you%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 8e) Defeat line 5 (you died)
	local match = string.match(line,"^You have been incapacitated by misadventure%.$");

	if (match ~= nil) then
		initiatorName = player.name;
		
		-- Update
		return 9,initiatorName;
	end
	
	-- 9) Revive lines --
	
	-- 9a) Revive line 1 (player revived)
	local initiatorName = string.match(line,"^(.*) has been revived%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9b) Revive line 2 (player succumbed)
	local initiatorName = string.match(line,"^(.*) has succumbed to .* wounds%.$");
	
	if (initiatorName ~= nil) then
	  initiatorName = string.gsub(initiatorName,"^The ","");
		initiatorName = string.gsub(initiatorName,"^the ","");
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9c) Revive line 3 (you were revived)
	local match = string.match(line,"^You have been revived%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 9d) Revive line 4 (you succumbed)
	local match = string.match(line,"^You succumb to your wounds%.$");
	
	if (match ~= nil) then
	  initiatorName = player.name;
	  
		-- Update
	  return 10,initiatorName;
	end
	
	-- 10) Temporary Morale line
	-- v4.0.1
	-- local amount = string.match(line,"You have lost ([%d,]*) points of temporary Morale!$");
	-- if (amount ~= nil) then
	--   -- Update
	--   
	--   
	--   return;
	-- end
	-- 
	--  (note for later: will need to also check off to see if corresponding damage was taken; always comes directly AFTER this line)
	
	
	-- if we reach here, we were unable to parse the line
	--  (note there is very little that isn't parsed; just a few combat events such as breaking CC,
	--     though some stuff is ignored, eg: dmg taken by pets)
	
end









--[[


Note: The following DOES NOT NEED TO BE ALTERED for the current version (but does need to exist).


The following is a (work-in-progress) list of specific default skills that will be tracked for
buff/debuff purposes in later versions of Combat Analysis. 

NB: If you do update this list, the skill names must identically match the text in the combat log in order for
these skills to be correctly detected.

The elements are indexed by: {Skill Name} = {Duration(s), {List of Conflicting Skill Names}}
 ... where conflicting skills are skills that get overidden by the application of this skill
			 (note this is only relevant for debuffs, buff tracking should allow us to determine overriding buffs automatically)
 ... durations are specified in seconds
			 (note that a duration of zero seconds indicates a toggle type skill)
	
]]--

-- 1) Burglar

-- a) Buffs
local BurglarBuffs = {}
-- b) Self Buffs
local BurglarSelfBuffs = {}
BurglarSelfBuffs["Touch and Go"] = {30}
-- Knives Out not included as it does not always appear in the log (if no mobs are nearby)
-- c) Debuffs
local BurglarDebuffs = {}
BurglarDebuffs["Reveal Weakness"] = {0, {}}
BurglarDebuffs["Trick: Disable"] = {30, {"Trick: Counter Defence", "Trick: Dust in the Eyes", "Trick: Enrage"}}
BurglarDebuffs["Trick: Counter Defence"] = {30, {"Trick: Disable", "Trick: Dust in the Eyes", "Trick: Enrage"}}
BurglarDebuffs["Trick: Dust in the Eyes"] = {30, {"Trick: Disable", "Trick: Counter Defence", "Trick: Enrage"}}
BurglarDebuffs["Trick: Enrage"] = {30, {"Trick: Disable", "Trick: Counter Defence", "Trick: Dust in the Eyes"}}
-- d) Crowd Control
local BurglarCrowdControl = {};
BurglarCrowdControl["Riddle"] = {30};

L["BurglarSkills"] = {BurglarBuffs,BurglarSelfBuffs,BurglarDebuffs,BurglarCrowdControl};

-- 2) Captain

-- a) Buffs
local CaptainBuffs = {}
-- b) Self Buffs
local CaptainSelfBuffs = {}
-- c) Debuffs
local CaptainDebuffs = {}
-- d) Crowd Control
local CaptainCrowdControl = {}

L["CaptainSkills"] = {CaptainBuffs,CaptainSelfBuffs,CaptainDebuffs,CaptainCrowdControl}

-- 3) Champion

-- a) Buffs
local ChampionBuffs = {}
-- b) Self Buffs
local ChampionSelfBuffs = {}
-- c) Debuffs
local ChampionDebuffs = {}
-- d) Crowd Control
local ChampionCrowdControl = {}

L["ChampionSkills"] = {ChampionBuffs,ChampionSelfBuffs,ChampionDebuffs,ChampionCrowdControl}

-- 4) Guardian

-- a) Buffs
local GuardianBuffs = {}
GuardianBuffs["Protection"] = {0}; -- NB: Need to check upgraded skill name, etc
GuardianBuffs["Protection by the sword"] = {0}; -- NB: Need to check upgraded skill name, etc
-- b) Self Buffs
local GuardianSelfBuffs = {}
GuardianSelfBuffs["Guardian's Ward"] = {10} -- NB: Need to check upgraded skill name, etc
GuardianSelfBuffs["Guardian's Ward Tactics"] = {10} -- NB: Need to check upgraded skill name, etc
-- c) Debuffs
local GuardianDebuffs = {}
GuardianDebuffs["Improved Overwhelm"] = {10, {}} -- NB: Need to check upgraded skill name, etc
-- Stagger not included due to its unreliableness
-- d) Crowd Control
local GuardianCrowdControl = {}

L["GuardianSkills"] = {GuardianBuffs,GuardianSelfBuffs,GuardianDebuffs,GuardianCrowdControl}

-- 5) Hunter

-- a) Buffs
local HunterBuffs = {}
-- b) Self Buffs
local HunterSelfBuffs = {}
-- c) Debuffs
local HunterDebuffs = {}
-- d) Crowd Control
local HunterCrowdControl = {}

L["HunterSkills"] = {HunterBuffs,HunterSelfBuffs,HunterDebuffs,HunterCrowdControl}

-- 6) Lore-Master

-- a) Buffs
local LoreMasterBuffs = {}
-- b) Self Buffs
local LoreMasterSelfBuffs = {}
-- c) Debuffs
local LoreMasterDebuffs = {}
-- d) Crowd Control
local LoreMasterCrowdControl = {}

L["LoreMasterSkills"] = {LoreMasterBuffs,LoreMasterSelfBuffs,LoreMasterDebuffs,LoreMasterCrowdControl}

-- 7) Minstrel

-- a) Buffs
local MinstrelBuffs = {}
-- b) Self Buffs
local MinstrelSelfBuffs = {}
-- c) Debuffs
local MinstrelDebuffs = {}
-- d) Crowd Control
local MinstrelCrowdControl = {}

L["MinstrelSkills"] = {MinstrelBuffs, MinstrelSelfBuffs,MinstrelDebuffs,MinstrelCrowdControl}

-- 8) Rune-Keeper

-- a) Buffs
local RuneKeeperBuffs = {}
-- b) Self Buffs
local RuneKeeperSelfBuffs = {}
-- c) Debuffs
local RuneKeeperDebuffs = {}
-- d) Crowd Control
local RuneKeeperCrowdControl = {}

L["RuneKeeperSkills"] = {RuneKeeperBuffs,RuneKeeperSelfBuffs,RuneKeeperDebuffs,RuneKeeperCrowdControl}

-- 9) Warden

-- a) Buffs
local WardenBuffs = {}
-- b) Self Buffs
local WardenSelfBuffs = {}
-- c) Debuffs
local WardenDebuffs = {}
-- d) Crowd Control
local WardenCrowdControl = {}

L["WardenSkills"] = {WardenBuffs,WardenSelfBuffs,WardenDebuffs,WardenCrowdControl}
